         .data
pila:  .space 63*4  @ Cima de la pila: pila
bpila: .space 4     @ Base (fondo) de la pila: bpila
ppila: .word 1      @ Puntero de pila

/*
    Operación de ejemplo: 
                0x20 + ((0x40 + 0x35) * 0x100) - 0x200 -> r0
*/

   .text
main:
   bl   initp
   mov  r0,#0x20
   bl   apila     @ apila 0x20
   mov  r0,#0x40
   bl   apila     @ apila 0x40
   mov  r0,#0x35
   bl   apila     @ apila 0x35

   bl   sumap     @ suma
   ldr  r0,=#0x100
   bl   apila     @ apila 0x100

   bl   multp     @ multiplica
   bl   sumap

   ldr  r0,=#0x200
   bl   apila     @ apila 0x200

   bl   swapp     @ swap
   bl   restap    @ resta

   bl   desap     @ desapila: resultado a r0

   wfi


/*
 initp: Subprograma para inicializar la pila
*/
initp:   
   ldr   r0,=bpila    @ dirección del fondo de la pila
   add   r0,r0,#4     @ una palabra más (bpila+4)  
   ldr   r1,=ppila
   str   r0,[r1]      @ inicializamos el puntero de pila
   mov   pc,lr        @ volvemos
   
/*
apila: Subprograma que almacena en la pila el contenido
   de r0. En caso de que no sea posible porque la pila
   está llena, se activa el indicador de cero (Z = 1)
   Si todo OK, Z = 0.

   El indicador Z para marcar pila llena resulta conveniente
   porque es el indicador que usamos en el código para
   completar o no la operación.
    
*/
apila:
   ldr   r1,=ppila
   ldr   r2,=pila
   ldr   r3,[r1]    @ r3: puntero de pila
   cmp   r3,r2      @ si iguales, pila llena y Z = 1
   beq   fina

   sub   r3,r3,#4   @ apilamos (y Z = 0)
   str   r0,[r3]
   str   r3,[r1]    @ actualizamos puntero de pila

fina:
   mov   pc,lr

/*
desap: Subprograma que saca la palabra de la cima de la pila y la
   deja en r0. En caso de que no sea posible porque la pila
   está vacía, se activa el indicador de signo (N=1) 
   Si todo OK, N=0.

   Elegimos el indicador N para marcar pila vacía porque resulta
   conveniente (es el indicador que usamos en el código para
   completar o no la operación).
*/
desap:
   ldr   r1,=ppila
   ldr   r2,=bpila
   ldr   r3,[r1]      @ r3: puntero de pila
   cmp   r2,r3        @ si r3 > r2, pila vacía y N = 1
   bmi   find

   ldr   r0,[r3]      @ desapilamos (y N = 0)
   add   r3,r3,#4
   str   r3,[r1]      @ actualizamos el puntero de pila

find:
   mov   pc,lr

/*
sumap:   Suma las dos palabras en la cima de la pila y las 
   sustituye por el resultado de la suma. Los indicadores 
   se activan en función del resultado de la suma.

   Si la pila está vacía o si solo hay un dato, no hace nada.
*/
sumap:
   ldr   r1,=ppila
   ldr   r2,=bpila
   ldr   r3,[r1]   @ r3: puntero de pila
   cmp   r2,r3
   bmi   fins      @ si r3 > r2, pila vacía
   beq   fins      @ si r3 = r2, solo un dato

   ldr   r0,[r3]   @ r0: suma de las dos palabras en la
   add   r3,r3,#4  @ cima de la pila, 
   ldr   r2,[r3]   @ y eliminamos el dato de sobra
   add   r0,r0,r2  @ 
   str   r0,[r3]   @ guardamos en la pila

   str   r3,[r1]   @ actualizamos el puntero de pila

fins:
   mov   pc,lr


/*
restap:   Resta las dos palabras en la cima de la pila y 
   las sustituye por el resultado de la suma. Los indica-
   dores se activan de acuerdo con el resultado de la resta

   Si la pila está vacía o si solo hay un dato, no hace nada.
*/
restap:
   ldr   r1,=ppila
   ldr   r2,=bpila
   ldr   r3,[r1]   @ r3: puntero de pila
   cmp   r2,r3
   bmi   finr      @ si r3 > r2, pila vacía
   beq   finr      @ si r3 = r2, solo un dato

   ldr   r0,[r3]   @ r0: resta de las dos palabras en la
   add   r3,r3,#4  @ cima de la pila, 
   ldr   r2,[r3]   @ y eliminamos el dato de sobra
   sub   r0,r0,r2  @ 
   str   r0,[r3]   @ guardamos en la pila
   str   r3,[r1]   @ actualizamos el puntero de pila

finr:
   mov    pc,lr

/*
multp:   Multiplica las dos palabras en la cima de la pila 
   y las sustituye por el resultado. Los indicadores 
   se activan de acuerdo con el resultado de la multiplicación.

   Si la pila está vacía o si solo hay un dato, no hace nada.
*/
multp:
   ldr   r1,=ppila
   ldr   r2,=bpila
   ldr   r3,[r1]   @ r3: puntero de pila
   cmp   r2,r3
   bmi   finm      @ si r3 > r2, pila vacía
   beq   finm      @ si r3 = r2, solo un dato

   ldr   r0,[r3]   @ r0: producto de las dos palabras en la
   add   r3,r3,#4  @ cima de la pila, 
   ldr   r2,[r3]   @ y eliminamos el dato de sobra
   mul   r0,r0,r2  @ 
   str   r0,[r3]   @ guardamos en la pila
   str   r3,[r1]   @ actualizamos el puntero de pila

finm:
   mov   pc,lr

/*
swapp:   Intercambia las dos palabras en la cima de la pila.

   Si la pila está vacía o si solo hay un dato, no hace nada.
*/
swapp:
   ldr   r1,=ppila
   ldr   r2,=bpila
   ldr   r3,[r1]    @ r3: puntero de pila
   cmp   r2,r3
   bmi   finw       @ si r3 > r2, pila vacía
   beq   finw       @ si r3 = r2, solo un dato

   ldr   r0,[r3]    @ r0, r2: para el intercambio
   ldr   r2,[r3,#4] @ 
   str   r0,[r3,#4] @ Intercambiamos
   str   r2,[r3]    @ 

finw:
   mov   pc,lr
   .end
